"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createPackageJson = void 0;
const fileutils_1 = require("./fileutils");
const object_sort_1 = require("nx/src/utils/object-sort");
const fs_1 = require("fs");
/**
 * Creates a package.json in the output directory for support to install dependencies within containers.
 *
 * If a package.json exists in the project, it will reuse that.
 */
function createPackageJson(projectName, graph, options) {
    const npmDeps = findAllNpmDeps(projectName, graph);
    // default package.json if one does not exist
    let packageJson = {
        name: projectName,
        version: '0.0.1',
        dependencies: {},
        devDependencies: {},
    };
    if ((0, fs_1.existsSync)(`${graph.nodes[projectName].data.root}/package.json`)) {
        try {
            packageJson = (0, fileutils_1.readJsonFile)(`${graph.nodes[projectName].data.root}/package.json`);
            if (!packageJson.dependencies) {
                packageJson.dependencies = {};
            }
            if (!packageJson.devDependencies) {
                packageJson.devDependencies = {};
            }
        }
        catch (e) { }
    }
    const rootPackageJson = (0, fileutils_1.readJsonFile)(`${options.root || '.'}/package.json`);
    Object.entries(npmDeps).forEach(([packageName, version]) => {
        var _a;
        if (((_a = rootPackageJson.devDependencies) === null || _a === void 0 ? void 0 : _a[packageName]) &&
            !packageJson.dependencies[packageName]) {
            packageJson.devDependencies[packageName] = version;
        }
        else {
            packageJson.dependencies[packageName] = version;
        }
    });
    packageJson.devDependencies && (packageJson.devDependencies = (0, object_sort_1.sortObjectByKeys)(packageJson.devDependencies));
    packageJson.dependencies && (packageJson.dependencies = (0, object_sort_1.sortObjectByKeys)(packageJson.dependencies));
    return packageJson;
}
exports.createPackageJson = createPackageJson;
function findAllNpmDeps(projectName, graph, list = {}, seen = new Set()) {
    var _a;
    if (seen.has(projectName)) {
        return list;
    }
    seen.add(projectName);
    const node = graph.externalNodes[projectName];
    if (node) {
        list[node.data.packageName] = node.data.version;
        recursivelyCollectPeerDependencies(node.name, graph, list);
    }
    else {
        // we are not interested in the dependencies of external projects
        (_a = graph.dependencies[projectName]) === null || _a === void 0 ? void 0 : _a.forEach((dep) => {
            if (dep.type === 'static' || dep.type === 'dynamic') {
                findAllNpmDeps(dep.target, graph, list, seen);
            }
        });
    }
    return list;
}
function recursivelyCollectPeerDependencies(projectName, graph, list = {}, seen = new Set()) {
    const npmPackage = graph.externalNodes[projectName];
    if (!npmPackage || seen.has(projectName)) {
        return list;
    }
    seen.add(projectName);
    const packageName = npmPackage.data.packageName;
    try {
        const packageJson = require(`${packageName}/package.json`);
        if (!packageJson.peerDependencies) {
            return list;
        }
        Object.keys(packageJson.peerDependencies)
            .map((dependencyName) => `npm:${dependencyName}`)
            .map((dependency) => graph.externalNodes[dependency])
            .filter(Boolean)
            .forEach((node) => {
            var _a, _b;
            if (!((_b = (_a = packageJson.peerDependenciesMeta) === null || _a === void 0 ? void 0 : _a[node.data.packageName]) === null || _b === void 0 ? void 0 : _b.optional)) {
                list[node.data.packageName] = node.data.version;
                recursivelyCollectPeerDependencies(node.name, graph, list, seen);
            }
        });
        return list;
    }
    catch (e) {
        return list;
    }
}
//# sourceMappingURL=create-package-json.js.map